//##
//##	File:		CustomDrag.c
//##
//##	Author:		Takenori Kabeya
//##
//##	Version:	GlueWindowINIT3 version 3.0.7
//##				CutomDrag.c	- 1.0.2
//##				May 11, 1996.
//##
//##	E-mail:		kabeya@kuic.kyoto-u.ac.jp
//##				GBH06222@niftyserve.or.jp
//##
//##	##	All comment is written in both English and Japanese.
//##		Of course, in Samurai English!
//##
#include	"CustomDrag.h"

#include	"GW_INIT.h"
#include	"MarqueePort.h"
#include	"DragMarquee.h"



//##
//##	Declarations : Prototypes Of Static Functions
//##
//##	X^eBbN֐̃vg^Cv錾
//##
static Boolean
TrackDragMarquee(
	WindowPtr		inWindowP,
	Point			inStartMousePt,
	RgnHandle		inLimitRgnH,
	RgnHandle		inMarqueeRgnH,
	Point			inMarqueeSize,
	Point*			ioMarqueePtP );

static void
AdjustMarqueePosition(
	WindowPeek		inWindowPeek,
	Point			inMarqueeSize,
	Point*			ioMarqueePtP,
	Boolean*		ioIsChangedHP,
	Boolean*		ioIsChangedVP );

static RgnHandle
CalcDragLimitRgn(
	const Rect*	inLimitRectP );

static void
MoveWindowAlongMarquee(
	WindowPtr	inWindowP,
	Point		inMarqueePt );



#pragma mark ===== Replace Functions =====



#pragma mark ........ Custom DragWindow ........
//
//	Custom drag-window-routine, traps Toolbox DragWindow().
//	Drag window's marquee like glue to the next window.
//
//	VXe{DragWindow()ƒuƎDragWindow()[`B
//	߂̃EBhEƂ悤ɃhbOB
//
//	MEMO:
//		Toolbox DragWindow() draws marquee 'as is' even if
//		the window looks non-rectangle.
//
//		I suppose it draw with FrameRgn() using window's
//		strucRgn.
//
//		And MoveWindow seems to need top-left of portRect.
//		But I want to treat windows by strucRgn.rgnBBox ( to
//		glue it ).
//
//		Then,
//
//		@	Marquee's frame is region in local coordinates.
//
//		@	And top-left of marquee's frame is global coordinates.
//
//		@	To use MoveWindow() with portRect, calc offset
//			between portRect and strucRgn.rgnBBox
//
//
//		ToolboxDragWindow()̓EBhĚ`󂪂ǂȌ`łA
//		̌`ɍ킹}[L[`B
//		炭A}[L[̌`strucRgnƓɂȂĂ
//		̂낤B
//		܂ASystem7ȍ~̃Rg[plȂǁA
//		ꍇAportRectcontRgn.rgnBBoxقȂĂ.
//		iSetOrigin()ƂȂjB
//
//		āAMoveWindow()portRect̍̍Ww肷
//		悤ɂȂĂ悤łB
//		AƂstrucRgn.rgnBBoxĂ
//		ǂiȂ킿AhbOĂ}[L[͂ދ`
//		ׂ̃EBhEstrucRgn.rgnBBoxƂĂ邩j
//		ŔfB
//
//		ȏ̂ƂƁAẄ̓O[oōsȒP
//		ȂƎv̂ŁA̕j߂B
//
//		@	}[L[̌`ێRegionstrucRgnRs[B
//			ŁARegion(0,0)iO[ojɂ܂ŁAĂ
//			ĂB
//		@	̂ŁA}[L[̍̍WϐO[o
//			WnŗpӂB
//		@	ŌMoveWindow()g߂ɁAportRectstrucRgn.rgnBBox
//			߂ĂA𗘗pă}[L[̍WA
//			MoveWindow()ɓnׂO[oW߂B
//
//	History:
//		12/Apr/1996
//			YĂ DisposeRgn( theMarqueeRgnH ); B
//
void
CustomDragWindow(
	WindowPtr	inWindowP,
	Point		inMousePt,
	Rect*		inLimitRectP )
{
	GrafPtr		theSavedPortP;
	GrafPort	theMarqueePort;	//	Not a pointer; //	̂pӁB
	RgnHandle	theLimitRgnH;
	RgnHandle	theMarqueeRgnH;
	Point		theMarqueePt;
	Point		theMarqueeSize;
	Boolean		isWindowMoved;
	
	//
	//	Prepare temporary graf port to draw marquee. set it to current port.
	//
	//	Jg|[gƂǂ̂
	//	Jg̃Ot|[gҔāA
	//	}[L[`߂̃Ot|[g쐬A
	//	JgɂB
	//
	GetPort( &theSavedPortP );
	OpenMarqueePort( &theMarqueePort );
	SetPort( &theMarqueePort );
	
	theLimitRgnH = CalcDragLimitRgn( inLimitRectP );
	
	//
	//	Setup frame of marquee in local coordinates.
	//
	//	}[L[̐ݒB
	//
	theMarqueeRgnH = GetNewDragMarquee( GetStrucRgn( inWindowP ),
										&theMarqueeSize, &theMarqueePt );
	
	//
	//	Show/Hide/Move marquee.
	//
	//	}[L[A\ړB
	//
	isWindowMoved = TrackDragMarquee( inWindowP, inMousePt, theLimitRgnH,
									  theMarqueeRgnH, theMarqueeSize,
									  &theMarqueePt );
	
	//
	//	Move window when mouse pointer moves.
	//
	//	EBhEړB
	//
	if( isWindowMoved ) {
		MoveWindowAlongMarquee( inWindowP, theMarqueePt );
	}
	
	//
	//	Restore port, and close a temporary marquee port.
	//
	SetPort( theSavedPortP );
	CloseMarqueePort( &theMarqueePort );
	
	DisposeRgn( theMarqueeRgnH );
	DisposeRgn( theLimitRgnH );
}



//
//	Track marquee like it glue to the next window.
//
//	ׂ̃EBhEɂ悤
//	}[L[𓮂[`B
//
Boolean
TrackDragMarquee(
	WindowPtr		inWindowP,
	Point			inStartMousePt,
	RgnHandle		inLimitRgnH,
	RgnHandle		inMarqueeRgnH,
	Point			inMarqueeSize,
	Point*			ioMarqueePtP )
{
	WindowPeek	theWindowPeek;
	Boolean		isChangedH;
	Boolean		isChangedV;
	Point		theNewMousePt;
	Boolean		isMoved = false;
	Point		theOldMousePt     = inStartMousePt;
	Point		theMarqueePt      = *ioMarqueePtP;
	Point		theMouseToMarquee = theMarqueePt;
	Boolean		isVisible = false;
	
	SubPt( theOldMousePt, &theMouseToMarquee );
	
	//
	//	First of all, draw marquee.
	//
	//	܂A}[L[`B
	//
	XorDragMarquee( inMarqueeRgnH, theMarqueePt, &isVisible );
	
	while( StillDown() ) {
		GetMouse( &theNewMousePt );
		LocalToGlobal( &theNewMousePt );
		
		//
		//	Don't redraw marquee when mouse pointer stay on the same point.
		//
		//	}EXړĂȂA
		//	}[L[`ȂȂB
		//
		if( !EqualPt( theNewMousePt, theOldMousePt ) ) {
			
			//
			//	Otherwise erase,
			//
			//	āA
			//
			if( isVisible ) {
				XorDragMarquee( inMarqueeRgnH, theMarqueePt, &isVisible );
			}
			
			//
			//	don't show marquee when mouse pointer not in limit region.
			//
			//	}EXJ[\͈͂ɓĂȂꍇA
			//	}[L[͕\ȂB
			//
			if( PtInRgn( theNewMousePt, inLimitRgnH ) ) {
			
				theMarqueePt = theNewMousePt;
				AddPt( theMouseToMarquee, &theMarqueePt );
				
				isChangedH = false;
				isChangedV = false;
				
				theWindowPeek = ( WindowPeek )FrontWindow();
				while( theWindowPeek != nil )	{
					if( theWindowPeek != ( WindowPeek )inWindowP ) {
					
						//
						//	Adjust marquee position like glue.
						//
						//	}[L[̈ʒũׂEBhEɂ悤ɕύXB
						//
						AdjustMarqueePosition( theWindowPeek, inMarqueeSize,
											   &theMarqueePt,
											   &isChangedH, &isChangedV );
						
						if( isChangedH && isChangedV ) {
							break;
						}
					}
					theWindowPeek = theWindowPeek->nextWindow;
				}
				
				//
				//	show again
				//
				//	ēx\B
				//
				XorDragMarquee( inMarqueeRgnH, theMarqueePt, &isVisible );
				
			}
			theOldMousePt = theNewMousePt;
		}
	}
	
	//
	//	At the end, erase marquee.
	//
	//	ŌɏB
	//
	if( isVisible ) {
		XorDragMarquee( inMarqueeRgnH, theMarqueePt, &isVisible );
		isMoved = true;
	}
	
	*ioMarqueePtP = theMarqueePt;
	
	return( isMoved );
}



//
//	EBhËʒuAׂ荇EBhE
//	悤ɐ䂷郋[`B
//
void
AdjustMarqueePosition(
	WindowPeek		inWindowPeek,
	Point			inMarqueeSize,
	Point*			ioMarqueePtP,
	Boolean*		ioIsChangedHP,
	Boolean*		ioIsChangedVP )
{
	Rect	theMarqueeRect;
	Point	theMarqueePt = *ioMarqueePtP;
	Boolean	isChangedH = *ioIsChangedHP;
	Boolean	isChangedV = *ioIsChangedVP;
	Rect	theWindowRect = ( **inWindowPeek->strucRgn ).rgnBBox;
	
	theMarqueeRect.left   = theMarqueePt.h;
	theMarqueeRect.top    = theMarqueePt.v;
	theMarqueeRect.right  = theMarqueePt.h + inMarqueeSize.h;
	theMarqueeRect.bottom = theMarqueePt.v + inMarqueeSize.v;
	
	//
	//	Each of following 'if' judges whether edge of marquee is close enough or not.
	//
	if( !isChangedH && ( Abs( theMarqueeRect.left - theWindowRect.left ) <= kPixelsToGlue ) ) {
		theMarqueePt.h = theWindowRect.left;
		isChangedH = true;	//	̃EBhE̍[Ɨׂ̃EBhE̍[߂ƂB
	}
	if( !isChangedH && ( Abs( theMarqueeRect.left - theWindowRect.right ) <= kPixelsToGlue ) ) {
		theMarqueePt.h = theWindowRect.right;
		isChangedH = true;	//	̃EBhE̍[Ɨׂ̃EBhẺE[߂ƂB
	}
	if( !isChangedH && ( Abs( theMarqueeRect.right - theWindowRect.left ) <= kPixelsToGlue ) ) {
		theMarqueePt.h = theWindowRect.left	- inMarqueeSize.h;
		isChangedH = true;	//	̃EBhẺE[Ɨׂ̃EBhE̍[߂ƂB
	}
	if( !isChangedH && ( Abs( theMarqueeRect.right - theWindowRect.right ) <= kPixelsToGlue ) ) {
		theMarqueePt.h = theWindowRect.right - inMarqueeSize.h;
		isChangedH = true;	//	̃EBhẺE[Ɨׂ̃EBhẺE[߂ƂB
	}
	if( !isChangedV && ( Abs( theMarqueeRect.top - theWindowRect.top ) <= kPixelsToGlue ) ) {
		theMarqueePt.v = theWindowRect.top;
		isChangedV = true;	//	̃EBhȄ[Ɨׂ̃EBhȄ[߂ƂB
	}
	if( !isChangedV && ( Abs( theMarqueeRect.top - theWindowRect.bottom ) <= kPixelsToGlue ) ) {
		theMarqueePt.v = theWindowRect.bottom;
		isChangedV = true;	//	̃EBhȄ[Ɨׂ̃EBhẺ[߂ƂB
	}
	if( !isChangedV && ( Abs( theMarqueeRect.bottom - theWindowRect.top ) <= kPixelsToGlue ) ) {
		theMarqueePt.v = theWindowRect.top - inMarqueeSize.v;
		isChangedV = true;	//	̃EBhẺ[Ɨׂ̃EBhȄ[߂ƂB
	}
	if( !isChangedV && ( Abs( theMarqueeRect.bottom - theWindowRect.bottom ) <= kPixelsToGlue ) ) {
		theMarqueePt.v = theWindowRect.bottom - inMarqueeSize.v;
		isChangedV = true;	//	̃EBhẺ[Ɨׂ̃EBhẺ[߂ƂB
	}
	
	*ioIsChangedHP = isChangedH;
	*ioIsChangedVP = isChangedV;
	*ioMarqueePtP = theMarqueePt;
}



//
//	Calc offset between portRect ( which is used by MoveWindow ) in global
//	coordinates and top-left of frame of the marquee in global coordinates.
//	And then call MoveWindow.
//
//
//	ۂɃEBhE}[L[̈ʒu
//	ړ郋[`B
//
void
MoveWindowAlongMarquee(
	WindowPtr	inWindowP,
	Point		inMarqueePt )
{
	Point		thePortPt;
	Point		theFrameToPort;
	
	//
	//	inWindowP->portRecẗʒuƁA
	//	inWindowP->strucRgn.rgnBBoẍʒuƂ
	//	vZ邽߂
	//	EBhẼ|[gZbgB
	//
	SetPort( inWindowP );
	
	thePortPt.h = inWindowP->portRect.left;
	thePortPt.v = inWindowP->portRect.top;
	
	LocalToGlobal( &thePortPt );
	
	theFrameToPort.h = thePortPt.h - GetStrucRgnBBox( inWindowP ).left;
	theFrameToPort.v = thePortPt.v - GetStrucRgnBBox( inWindowP ).top;
	
	//
	//	vZpāAinWindowP->portRect̍
	//	WO[oɒAۂɃEBhE
	//	ړB
	//
	MoveWindow( inWindowP, inMarqueePt.h + theFrameToPort.h,
						   inMarqueePt.v + theFrameToPort.v, false );
}



//
//	Calc the region which allows user to move marquee.
//	( To be exact, the region where marquee goes with
//	  mouse pointer. )
//
//	When the user have two or more monitors,
//	user cannot drag the window outside main monitor
//	with some programmer's code.
//
//	Judge the programmer's limit region is enough large
//	than gray area of main monitor.
//
//	if enough large, then allow user to second monitor,
//	otherwise restrict.
//
//	'enough' is 5 pixels. In Inside Macintosh,
//	4 pixels seems to good, but DragWindow uses 5 or more
//	pixels. I don't know what is the best.
//
//	}[L[̈ړ𐧌͈́imɂ̓}EX
//	|C^Ƀ}[L[Ǐ]͈́jݒ肷
//	[`B
//
//	}`j^̏ꍇÂ܂܂LimitRectł
//	hbOłȂB
//
//	qd.screenBits.boundsA4sNZk
//	i4̍́AInside MacɁuLimitRect
//	ʏAqd.screenBits.bounds4sNZ
//	ȏ㏬vƏĂAƂAق
//	HĂȂ́jAƁAĂLimitRect
//	̕܊֌W𒲂ׂB
//
//	ALimitRect̕傫΁AGrayRgn
//	4sNZk[WgpBŃ}`
//	j^ɃhbOłB
//
//	΁A܂AǏIȃhbOƂ݂ȂA
//	LimitRect[Wɕϊ̂pB
//
//	History:
//		11/Mar/1996
//		}`j^̎AhbOłȂꍇB
//		̓Iɂ̓j[o[̍A炩LimitRect
//		k߂ēnAvP[VȀꍇA
//		ǏIȃhbOƂ݂ȂĂ܂悤Ȃ̂ŁA
//		j[o[̍l悤ɕύXB
//
//		12/Apr/1996
//		ƂƂA܂܂hbOłȂꍇ
//		킯ŁAk镝5sNZɕύXB
//
RgnHandle
CalcDragLimitRgn(
	const Rect*	inLimitRectP )
{
	Rect		theBigRect;
	RgnHandle	theTempRgnH;
	RgnHandle	theGrayRgnH   = GetGrayRgn();
	RgnHandle	theLimitRgnH  = NewRgn();
	SInt16		theMBarHeight = LMGetMBarHeight();
	
	theBigRect = gQDGlobals.screenBits.bounds;
	InsetRect( &theBigRect, 5, 5 );
	theBigRect.top += theMBarHeight;
	
	//
	//	ĂLimitRect̕AfXNgbv4sNZ
	//	k̂傫ꍇA
	//
	if( inLimitRectP->left <=  theBigRect.left &&
		inLimitRectP->top  <=  theBigRect.top  &&
		theBigRect.right   <= inLimitRectP->right &&
		theBigRect.bottom  <= inLimitRectP->bottom ) {
		
		CopyRgn( theGrayRgnH, theLimitRgnH );
		InsetRgn( theLimitRgnH, 5, 5 );
	}
	//
	//	łȂāAfXNgbv̕傫āAǏIȃhb
	//	ȌꍇA
	//
	else {
		theTempRgnH = NewRgn();
		CopyRgn( theGrayRgnH, theTempRgnH );
		InsetRgn( theTempRgnH, 5, 5 );
		RectRgn( theLimitRgnH, inLimitRectP );
		SectRgn( theLimitRgnH, theTempRgnH, theLimitRgnH );
		DisposeRgn( theTempRgnH );
	}
	
	return( theLimitRgnH );
}
